import yaml
import importlib
import sys
import traceback
import multiprocessing

# Mapping of package names to their import names
package_import_mapping = {
    'pyyaml': 'yaml',
    'scikit-learn': 'sklearn',
    'python-eccodes': 'eccodes',
    'python-cdo': 'cdo',
    'pynco': 'nco',
    'netcdf4': 'netCDF4',
    'r-circlize': 'rpy2.robjects.packages',
    'jupyter': 'jupyter',
    'ipykernel': 'ipykernel',
    'xclim': 'xclim',
    'holoviews': 'holoviews',
    'zarr': 'zarr',
    'ipywidgets': 'ipywidgets',
    'tqdm': 'tqdm',
    'gdal': 'osgeo.gdal',
    'plotly': 'plotly',
    'scipy': 'scipy',
    'statsmodels': 'statsmodels',
    'geopandas': 'geopandas',
    'rtree': 'rtree',
    'eccodes': 'eccodes',
    'filelock': 'filelock',
    'configargparse': 'configargparse',
    'mpi4py': 'mpi4py'
}

# List of non-Python packages
non_python_packages = ['mpich', 'python']

def load_modules_from_yaml(yaml_path):
    with open(yaml_path, 'r') as file:
        env_config = yaml.safe_load(file)
    modules = set()  # Use a set to store unique module names
    for dependency in env_config['dependencies']:
        if isinstance(dependency, str):
            module_name = dependency.split('=')[0]  # Handle version specs
            if module_name not in non_python_packages:
                modules.add(package_import_mapping.get(module_name, module_name))
        elif isinstance(dependency, dict) and 'pip' in dependency:
            for pip_package in dependency['pip']:
                module_name = pip_package.split('=')[0]
                modules.add(package_import_mapping.get(module_name, module_name))
    return modules

def test_module(module):
    try:
        importlib.import_module(module)
        print(f"Module '{module}' is installed correctly.")
    except ImportError:
        print(f"Module '{module}' is NOT installed.")
    except Exception as e:
        print(f"An error occurred while testing module '{module}': {e}")
        traceback.print_exc()

def test_modules(modules):
    for module in modules:
        process = multiprocessing.Process(target=test_module, args=(module,))
        process.start()
        process.join()

    print("Test completed.")

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python test_conda_env.py <path_to_yaml>")
        sys.exit(1)

    yaml_path = sys.argv[1]
    modules = load_modules_from_yaml(yaml_path)
    test_modules(modules)
